/**
 * Copyright (C) 2003-2004 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.util;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.sql.DataSource;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import sync4j.exchange.DataAccessException;

/**
 * This class provide methods
 * to access database source
 *
 * @author Fabio Maggi @ Funambol
 * @version $Id: DataAccess.java,v 1.3 2004/08/04 17:01:45 fabius Exp $
 **/
public class DataAccess {

    // ------------------------------------------------------------------------- Constants

    private static final String JNDI_DATASOURCE_NAME = "java:/jdbc/sync4j";

    // ------------------------------------------------------------------------- Private data

    private String jndiDataSourceName = null;

    // ------------------------------------------------------------------------- Protected data

    /**
     * Data store to use for database connections
     */
    protected transient DataSource dataSource = null;

    // ------------------------------------------------------------------------- Costructor

    /** Creates a new instance of DataAccess */
    public DataAccess()
    throws DataAccessException {
        setJndiDataSourceName(JNDI_DATASOURCE_NAME);
    }

    // ------------------------------------------------------------------------- Public methods

    public void setJndiDataSourceName(String jndiDataSourceName)
    throws DataAccessException {

        this.jndiDataSourceName = jndiDataSourceName;

        if (jndiDataSourceName == null) {
            dataSource = null;
        }

        try {
            InitialContext ctx = new InitialContext();
            dataSource = (DataSource) ctx.lookup(jndiDataSourceName);

        } catch (NamingException e) {
            throw new DataAccessException ("Data source "
            + jndiDataSourceName
            + " not found "
            + e.toString()
            );
        }
    }

    /**
     * @return database connection
     *
     * @throws DataAccessException
     **/
    public Connection getConnection ()
    throws DataAccessException {

        Connection con = null;

        try {

            con = this.dataSource.getConnection();

        } catch (SQLException e) {
            throw new DataAccessException
                ("Error create connection: " + e.getMessage());
        }

        return con;

    }

    /**
     * cleanUp database connection
     *
     * @param con coonection
     * @param ps  preparement statement
     * @param rs  resultset
     *
     * @throws DataAccessException
     **/
    public void cleanUp (Connection        con ,
                         PreparedStatement ps  ,
                         ResultSet         rs  )
    throws DataAccessException {

        try {

            if (con != null) {
                con.close();
                con = null;
            }

            if (ps != null) {
                ps.close();
                ps = null;
            }

            if (rs != null) {
                rs.close();
                rs = null;
            }

        } catch (SQLException e) {
            throw new DataAccessException
                ("Error cleanUp connection: " + e.getMessage());
        }

    }

}